# ---------------------------------------------
# Signal, Spectre/Cepstre, Sonagram, Cochleagram & infos
# PRAAT script with cochleagram
# Pascal Gaillard 2012
# pascal.gaillard@univ-tlse2.fr
# version 1.1 (sept 20012)
# ---------------------------------------------
# Output : PDF or EPS file


# --------------- Parameters
form Sonagram & spectrum parameters
	sentence Max_Sona_Frequency 12000
	sentence x_axis_label Time (s)
	real max_view 0
	sentence y_axis_label Frequency (Hz)
	real window_lenght 0.03
	integer dynamic_range  35
	integer pre_emphasis  6
	real dynamic_compression  0.6
	boolean cepstrum_option 1
	optionmenu type_of_image: 4
		option Image (copy to clipboard)
		option Vectoriel (copy to clipboard)
		option PostScript (write on disk)
		option PDF (write on disk)
	sentence analysis_output_directory ~/Desktop/


endform

#########################
# PostScript settings... yes finest automatic yes
#########################


# -----------------default analysis parameters
# window_lenght = 0.002
time_step = 0.002
f_step = 20

# ------------------display parameters
max_db = 100
# dynamic_range = 35
# pre_emphasis = 6
# dynamic_compression = 0.6


# ----------------read the selected filename
soundname$ = selected$("Sound")

# ----------------read the total duration and sampling frequency of the sound
	select Sound 'soundname$'
	total_duration = Get total duration
	sampling_frequency = Get sampling frequency

# ----------------erase praat picture screen
	Erase all


# ----------------set the font and color
	Times
	Font size... 10
	Black
	Line width... 0.5

# ---------------- waveform
# Define size and position of waveform (by specifying grid coordinates)
	select Sound 'soundname$'
	Viewport... 0 6 0 2
	Maroon
	Draw... 0 max_view 0 0 no curve
	
# ----------------waveform labels creation
	# Label x axis 
	# Text bottom... yes 'x_axis_label$' 
	# Marks bottom every... 1 0.1 yes yes no
	Draw inner box
 
# ---------------- spectrogram process
	select Sound 'soundname$'
	To Spectrogram... 'window_lenght'  'Max_Sona_Frequency$' 'time_step' 'f_step' Gaussian

# ---------------- spectrogram paint
	Viewport... 0 6 1.5 5
	Paint... 0 max_view 0 0 max_db yes dynamic_range pre_emphasis dynamic_compression no

# ---------------- spectrgram labels
# Label y axis
	One mark left... 0 no no yes
	Marks left every... 1 1000 yes yes no
	
	###
	# temp and freq resolution
	####
	respoint = sampling_frequency * window_lenght
	respoint = round(respoint)

	# res = 1 / window_lenght
	res = sampling_frequency / respoint
	res = round(res)

	# problem with vertical label in screen copy
	Text left... yes 'y_axis_label$' - res: 'res' (Hz)
	
# label Hz axis
	Viewport... 0 6 1.5 5

# Label x axis 
	Text bottom... yes 'x_axis_label$' - res: 'window_lenght' (s) - 'respoint' (pts)
	One mark bottom... total_duration  no yes yes 'total_duration:2'
# change temp label according to duration
	if total_duration <= 0.1
		Marks bottom every... 1 0.02 yes yes no
	elsif total_duration <= 0.5
		Marks bottom every... 1 0.05 yes yes no
	elsif total_duration <= 1
		Marks bottom every... 1 0.2 yes yes no
	elsif total_duration <= 5
		Marks bottom every... 1 0.5 yes yes no
	elsif total_duration <= 20
		Marks bottom every... 1 1 yes yes no
	elsif total_duration <= 61
		Marks bottom every... 1 5 yes yes no

	else
		Marks bottom every... 1 10 yes yes no
	endif	

	Draw inner box

# --------------- sound pressure on spectrogram
	select Sound 'soundname$'
	To Intensity... 100 0 yes
	select Intensity 'soundname$'
	# intensity on spectrogram
	Viewport... 0 6 1.5 5
	# intensity on waveform
	# Viewport... 0 6 0 2
	Draw... 0 max_view 0 0 no


# ----------------spectrum process
select Sound 'soundname$'
To Spectrum... 1

# ----------------spectrum display
Viewport... 6 12 0 5
mini_power_db = 0
max_power_db = 0
if cepstrum_option = 1
	Cepstral smoothing... 250
endif
Draw (log freq)... 10 'Max_Sona_Frequency$' 0 0 1

# ----------------process spectrum information (like center of gravity)
select Spectrum 'soundname$'

centregravity = Get centre of gravity... 2.0
standarddev = Get standard deviation... 2.0

centregravity = round('centregravity')
standarddev = round('standarddev')

# ---------------- cochleagram creation
select Sound 'soundname$'
# parameters are : time step (s), default = 0.01
#                           Frequency resolution (bark), default = 0.1
#                           Window length (s), default = 0.03
#                           Forward-masking time (s), default = 0.03
# To Cochleagram... 0.01 0.1 0.03 0.03
To Cochleagram... 0.01 0.1 0.03 0.03
Viewport... 0 6 4.75 8 
Paint... 0 0 1


# ----------------write title and informations

Font size... 18
Viewport... 9 9.25 5.5 6
Maroon
Text top... no Sound object : 'soundname$' 

Font size... 14
Black


Viewport... 6.5 6.75 6 6.25
Text... 0 Left 0 top Sampling frequency : 
Viewport... 6.5 6.75 6.25 6.5
Text... 0 Left 0 top Total duration: 
Viewport... 6.5 6.75 6.5 6.75
Text... 0 Left 0 top Centre of gravity :
Viewport... 6.5 6.75 6.75 7
Text... 0 Left 0 top Standard deviation :

Viewport... 8.5 8.75 6 6.25
Text... 0 Left 0 top 'sampling_frequency' Hz
Viewport... 8.5 8.75 6.25 6.5
Text... 0 Left 0 top 'total_duration:2' s
Viewport... 8.5 8.75 6.5 6.75
Text... 0 Left 0 top 'centregravity' Hz
Viewport... 8.5 8.75 6.75 7
Text... 0 Left 0 top 'standarddev' Hz

Font size... 10
Black

Maroon
# ----------------labels on each figure
	Select outer viewport... 5 6 0 0.65
	Text... 0 Right 0 Top  Waveform
	Select outer viewport... 5 6 1.75 2
	Text... 0 Right 0 Top  Spectrogram
	Select outer viewport... 5 6 5 5.25
	Text... 0 Right 0 Top  Cochleagram
	Viewport... 6.35 7 0 0.65
	Text... 0 Left 0 Top  Spectrum
Black





# ------------ copy to
	Viewport... 0 12 0 8
	if type_of_image$	= "Image (copy to clipboard)"
		Copy screen image to clipboard
		echo Screen image is in the clipboard
	endif	
	if type_of_image$	= "Vectoriel (copy to clipboard)"
		Copy to clipboard
		echo Vectored draw is in the clipboard
	endif	
	if type_of_image$	= "PostScript (write on disk)"
		Write to EPS file... 'analysis_output_directory$''soundname$'-analyse.eps
		echo File "'soundname$'-analyse.eps" was written into "'analysis_output_directory$'" directory
	endif	
	if type_of_image$	= "PDF (write on disk)"
		Write to PDF file... 'analysis_output_directory$''soundname$'-analyse.pdf
		echo File "'soundname$'-analyse.pdf" was written into "'analysis_output_directory$'" directory
	endif	
		
# ------------  remove
	select Spectrogram 'soundname$'
	Remove
	select Spectrum 'soundname$'
	Remove
if cepstrum_option = 1
	select Spectrum 'soundname$'
	Remove
endif
	select Intensity 'soundname$'
	Remove
	select Cochleagram 'soundname$'
	Remove
	select Sound 'soundname$'
